CIFUI = ISCollapsableWindow:derive("CIFUI");
CIFUI.instance = nil;
CIFUI.SMALL_FONT_HGT = getTextManager():getFontFromEnum(UIFont.Small):getLineHeight()
CIFUI.MEDIUM_FONT_HGT = getTextManager():getFontFromEnum(UIFont.Medium):getLineHeight()
CIFUI.containerItemsCache = nil
CIFUI.actionInProgress = false

local width = 300
local height = 420
local grabBtn = getTexture("media/textures/GrabUI.png")
local grabBtnX = 210
local posX = 0
local posY = 0
local limit = 250
local getALL = false
local lastValidFilter = ""

function CIFUI:show(container,title)
    local square = getPlayer():getSquare()
    getALL = false
    lastValidFilter = ""
    posX = square:getX()
    posY = square:getY()
    if CIFUI.instance==nil then
        CIFUI.instance = CIFUI:new (0, 0, width, height,container,title);
        CIFUI.instance.container = container
        CIFUI.instance:initialise();
        CIFUI.instance:instantiate();
    end
    CIFUI.instance.pinButton:setVisible(false)
    CIFUI.instance.collapseButton:setVisible(false)
    CIFUI.instance:addToUIManager();
    CIFUI.instance:setVisible(true);
    return CIFUI.instance;
end

function CIFUI:update()
    local player = self.player
	if player:DistTo(posX, posY) > 2 then
		self:close()
    end
end

function CIFUI:onFilterChange()
    local filterText = string.trim(self.parent.filterEntry:getInternalText())
    if string.len(filterText) < 2 then return end
    self.parent:filter()
end

function CIFUI:filter()
    local items = self:reloadItems()
    local filterText = string.trim(self.filterEntry:getInternalText())
    filterText = string.lower(filterText)
    lastValidFilter = filterText
    local containertems = self.containertems.items
    self.containertems:clear()
    local count = 0
    for i=0, items:size()-1 do
        local item = items:get(i)
        if item then
            if string.contains(string.lower(item:getName()), filterText) then
                if count <= limit then
                    self.containertems:addItem(item:getFullType(),item);
                end
                count = count + 1
            end
        end
    end
    local label = getText("IGUI_CIF_Total").." : " ..count
    self.totalFound:setName(label)
end

local currentTooltip = nil
function CIFUI:toggleTooltip(show,item)
    if item then
        if not currentTooltip then
            currentTooltip = ISToolTipInv:new(item)
            currentTooltip:initialise();
        else
            currentTooltip:addToUIManager()
            currentTooltip:setItem(item)
            currentTooltip:setVisible(true)
            currentTooltip:setOwner(self)
            currentTooltip:render();
        end
    end
    if not show and currentTooltip then
        currentTooltip:removeFromUIManager()
        currentTooltip:setVisible(false)
    end
end

function CIFUI:onMouseMove(dx, dy)
    self.mouseOver = true;
	if self.moving then
		self:setX(self.x + dx);
		self:setY(self.y + dy);
		self:bringToTop();
	end
    self:toggleTooltip(false)
end

function CIFUI:reloadItems()
    return self.container:getItems()
end

function CIFUI:onMouseDownContainerItem(x, y)
    ISScrollingListBox.onMouseDown(self,x, y)
    local list = self
    local rowIndex = list:rowAt(list:getMouseX(), list:getMouseY())
    if rowIndex < 1 then return end
    local mouseX = self:getMouseX()
    if mouseX > grabBtnX then
        local item = self.items[rowIndex].item
        local container = item:getContainer()
        local player = self.parent.player
        local playerInv = player:getInventory()
        self.parent.actionInProgress = true
        local action = ISInventoryTransferAction:new(player, item, container, playerInv)
        action:setOnComplete(CIFUI.filter,self.parent)
        ISTimedActionQueue.add(action)
    end
end

function CIFUI:onMouseMoveContainerItem(dx, dy)
    local list = self
    if list:isMouseOver() then
        local rowIndex = list:rowAt(list:getMouseX(), list:getMouseY())
        if not rowIndex then self.parent:toggleTooltip(false) return end
        local selectedRow = list.items[rowIndex]
        if not selectedRow then self.parent:toggleTooltip(false) return end
        self.parent:toggleTooltip(true,selectedRow.item)
    end
end

function CIFUI:doDrawItem(y, item, alt)
    local baseItemDY = 0
    if item.item:getName() then
        baseItemDY = self.SMALL_FONT_HGT
        item.height = self.itemheight + baseItemDY
    end

    if y + self:getYScroll() >= self.height then return y + item.height end
    if y + item.height + self:getYScroll() <= 0 then return y + item.height end

    local a = 0.9;
    self:drawRectBorder(0, (y), self:getWidth(), item.height - 1, a, self.borderColor.r, self.borderColor.g, self.borderColor.b);

    if self.selected == item.index then
        self:drawRect(0, (y), self:getWidth(), item.height - 1, 0.3, 0.7, 0.35, 0.15);
    end

    self:drawText(item.item:getName(), 32, y + 10, 1, 1, 1, a, UIFont.Small);
    self:drawTextureScaledAspect(item.item:getTex(), 8, y+8, 20, 20, 1, 1, 1, 1)
    self:drawTextureScaledAspect(grabBtn, grabBtnX, y + 7, 20, 20, 1, 1, 1, 1)

    return y + item.height;
end

function CIFUI:createChildren()
    ISCollapsableWindow.createChildren(self);

    local th = self:titleBarHeight();
    local rh = self.resizable and self:resizeWidgetHeight() or 0

    self.filterLabel = ISLabel:new(20, 50, 1,getText("IGUI_CIF_Search"),1,1,1,1,UIFont.Small, true);
    self:addChild(self.filterLabel);

    local width = ((self.width/2) - getTextManager():MeasureStringX(UIFont.Small, getText("IGUI_CIF_Search"))) - 100;
    self.filterEntry = ISTextEntryBox:new("", getTextManager():MeasureStringX(UIFont.Small,getText("IGUI_CIF_Search")) + 30, 40, 120, 1);
    self.filterEntry:initialise();
    self.filterEntry:instantiate();
    self.filterEntry:setText("");
    self.filterEntry:setClearButton(true);
    self.filterEntry.onTextChange = CIFUI.onFilterChange
    self:addChild(self.filterEntry);

    self.containertems = ISScrollingListBox:new(25, 70, 250, 300);
    self.containertems:initialise();
    self.containertems:instantiate();
    self.containertems:setAnchorRight(false)
    self.containertems:setAnchorBottom(true)
    self.containertems.font = UIFont.NewSmall;
    self.containertems.itemheight = self.MEDIUM_FONT_HGT;
    self.containertems.selected = 1;
    self.containertems.joypadParent = self;
    self.containertems.drawBorder = false;
    self.containertems.SMALL_FONT_HGT = self.SMALL_FONT_HGT
    self.containertems.MEDIUM_FONT_HGT = self.MEDIUM_FONT_HGT
    self.containertems.doDrawItem = CIFUI.doDrawItem;
    self.containertems.onMouseMove = CIFUI.onMouseMoveContainerItem;
    self.containertems.onMouseDown = CIFUI.onMouseDownContainerItem;
    self:addChild(self.containertems);

    local container = self.container
    local items = container:getItems()
    local count = 0
    for i=0, items:size()-1 do
        local item = items:get(i)
        if item then
            if count <= limit then
                self.containertems:addItem(item:getFullType(),item);
            end
            if count >= limit then break end
            count = count + 1
        end
    end
    local label = getText("IGUI_CIF_Total").." : " ..items:size()
    self.totalFound = ISLabel:new(200, 43, CIFUI.SMALL_FONT_HGT, label, 1, 1, 1, 1, UIFont.Small, true)
    self:addChild(self.totalFound);

    self.grabQuantity = ISTextEntryBox:new("0", 30, 380, 50, 20);
    self.grabQuantity.font = UIFont.Medium
	self.grabQuantity:initialise();
	self.grabQuantity:instantiate();
	self.grabQuantity:setOnlyNumbers(true);
	self:addChild(self.grabQuantity);

    self.grabButton = ISButton:new(100, 380, 60,25,getText("IGUI_CIF_Grab"),self, CIFUI.grabButton);
    self.grabButton:initialise()
    self.grabButton.enable = true
    self:addChild(self.grabButton);

    self.grabAllButton = ISButton:new(200, 380, 60,25,getText("IGUI_CIF_GrabAll"),self, CIFUI.grabButton);
    self.grabAllButton.internal = "ALL"
    self.grabAllButton:initialise()
    self.grabAllButton.enable = true
    self:addChild(self.grabAllButton);

    self:bringToTop()
end

function CIFUI:transferItems(quantity)
    local filterText = lastValidFilter
    local container = self.container
    local containerItems = container:getItems()
    local player = self.player
    local playerInv = player:getInventory()
    self.actionInProgress = true
    local count = 0
    for i=0, containerItems:size()-1 do
        local item = containerItems:get(i)
        if item then
            if getALL then
                if string.contains(string.lower(item:getName()), filterText) then
                    ISTimedActionQueue.add(ISInventoryTransferAction:new(player, item, item:getContainer(), playerInv))
                end
            else
                if string.contains(string.lower(item:getName()), filterText) then
                    if count < quantity then
                        ISTimedActionQueue.add(ISInventoryTransferAction:new(player, item, item:getContainer(), playerInv))
                        count = count + 1
                    end
                end
                if count > quantity then break end
            end
        end
    end
    getALL = false
end

function CIFUI:grabButton(button)
    if #self.containertems.items <=0 then return end
    local quantity = tonumber(self.grabQuantity:getInternalText())
    if button.internal == "ALL" then
        getALL = true
        quantity = 1
    end
    if not quantity then return end
    if quantity <=0 then return end  
    if not quantity == math.floor(quantity) then return end
    self:transferItems(quantity)
    self:close()
end

function CIFUI:render()
    ISCollapsableWindow.render(self);
    local actionQueue = ISTimedActionQueue.getTimedActionQueue(self.player)
    local currentAction = actionQueue.queue[1]
    if not currentAction then self.actionInProgress = false return end
    if currentAction.Type == "ISInventoryTransferAction" and self.actionInProgress then
        self:drawProgressBar(78, 21, 140, 15, currentAction.action:getJobDelta(), self.fgBar)
        self:drawTextCentre(getText("IGUI_TakingFromContainer"), 148, 21, 1,1,1,1, UIFont.Small);
    else
        self.actionInProgress = false
    end
end

function CIFUI:close()
	ISCollapsableWindow.close(self);
    CIFUI.instance = nil
    self:removeFromUIManager()
end

function CIFUI:new(x, y, width, height,container,title)
    local o = {}
    if x == 0 and y == 0 then
        x = (getCore():getScreenWidth() / 2) - (width / 2);
        y = (getCore():getScreenHeight() / 2) - (height / 2);
    end
    o = ISCollapsableWindow:new(x, y, width, height);
    setmetatable(o, self)
    o.fgBar = {r=0, g=0.6, b=0, a=0.7 }
    self.__index = self
    o.player = getPlayer()
    o.title = title;
    o.resizable = false;
    return o
end